Exception: Graphiti::Errors::PolymorphicResourceChildNotFound

Inherits:
Base
  • Object
show all
Defined in:
lib/graphiti/errors.rb

Instance Method Summary collapse

Constructor Details

#initialize(resource_class, type: nil, model: nil) ⇒ PolymorphicResourceChildNotFound

Returns a new instance of PolymorphicResourceChildNotFound.



503
504
505
506
507
# File 'lib/graphiti/errors.rb', line 503

def initialize(resource_class, type: nil, model: nil)
  @resource_class = resource_class
  @model = model
  @type = type
end

Instance Method Details

#messageObject



509
510
511
# File 'lib/graphiti/errors.rb', line 509

def message
  @model ? model_message : type_message
end

#model_messageObject



513
514
515
516
517
518
519
520
521
522
# File 'lib/graphiti/errors.rb', line 513

def model_message
  <<~MSG
    #{@resource_class}: Tried to find Resource subclass with model #{@model.class}, but nothing found!

    Make sure all your child classes are assigned and associated to the right models:

    # One of these should be assocated to model #{@model.class}:
    self.polymorphic = ['Subclass1Resource', 'Subclass2Resource']
  MSG
end

#type_messageObject



524
525
526
527
528
529
530
531
532
533
# File 'lib/graphiti/errors.rb', line 524

def type_message
  <<~MSG
    #{@resource_class}: Tried to find Resource subclass with model #{@model.class}, but nothing found!

    Make sure all your child classes are assigned and associated to the right models:

    # One of these should be assocated to model #{@model.class}:
    self.polymorphic = ['Subclass1Resource', 'Subclass2Resource']
  MSG
end