Exception: Graphiti::Errors::PolymorphicSideloadTypeNotFound

Inherits:
Base
  • Object
show all
Defined in:
lib/graphiti/errors.rb

Instance Method Summary collapse

Constructor Details

#initialize(sideload, name) ⇒ PolymorphicSideloadTypeNotFound

Returns a new instance of PolymorphicSideloadTypeNotFound.



620
621
622
623
# File 'lib/graphiti/errors.rb', line 620

def initialize(sideload, name)
  @sideload = sideload
  @name = name
end

Instance Method Details

#messageObject



625
626
627
628
629
630
631
632
633
634
635
636
637
638
# File 'lib/graphiti/errors.rb', line 625

def message
  <<~MSG
    #{@sideload.parent_resource}: Tried to find a Resource with type '#{@name.inspect}', but did not find one!

    This is because either a Resource with that type doesn't exist, or it's not registered on the sideload. The below example shows how to register a Resource with this sideload. Make sure one of the registered Resources has type '#{@name.inspect}'

    polymorphic_belongs_to #{@sideload.name.inspect} do
      group_by(#{@sideload.grouper.field_name.inspect}) do
        on(:foo)
        on(:foo).belongs_to :foo, resource: FooResource # (long-hand example)
      end
    end
  MSG
end