Exception: Graphiti::Errors::UndefinedIDLookup

Inherits:
Base
  • Object
show all
Defined in:
lib/graphiti/errors.rb

Instance Method Summary collapse

Constructor Details

#initialize(resource_class) ⇒ UndefinedIDLookup

Returns a new instance of UndefinedIDLookup.



416
417
418
# File 'lib/graphiti/errors.rb', line 416

def initialize(resource_class)
  @resource_class = resource_class
end

Instance Method Details

#messageObject



420
421
422
423
424
425
426
# File 'lib/graphiti/errors.rb', line 420

def message
  <<~MSG
    Tried to resolve #{@resource_class} with an :id filter, but the filter was nil.
    This can result in unscoping a query, which can cause incorrect values to be
    returned which may or may not bypass standard access controls.
  MSG
end