Class: Graphiti::SpecHelpers::ContextProxy
- Defined in:
- lib/graphiti/spec_helpers/rspec.rb
Instance Method Summary collapse
-
#current_user ⇒ Object
variables defined in the rspec context should remain lazy.
-
#initialize(proxied, *args) ⇒ ContextProxy
constructor
A new instance of ContextProxy.
- #params ⇒ Object
Constructor Details
#initialize(proxied, *args) ⇒ ContextProxy
Returns a new instance of ContextProxy.
7 8 9 10 |
# File 'lib/graphiti/spec_helpers/rspec.rb', line 7 def initialize(proxied, *args) @__proxied = proxied super(*args) end |
Instance Method Details
#current_user ⇒ Object
variables defined in the rspec context should remain lazy
13 14 15 |
# File 'lib/graphiti/spec_helpers/rspec.rb', line 13 def current_user super || __proxied_current_user end |
#params ⇒ Object
17 18 19 |
# File 'lib/graphiti/spec_helpers/rspec.rb', line 17 def params super || __proxied_params end |