Exception: Graphiti::Errors::PolymorphicResourceChildNotFound
Instance Method Summary collapse
-
#initialize(resource_class, type: nil, model: nil) ⇒ PolymorphicResourceChildNotFound
constructor
A new instance of PolymorphicResourceChildNotFound.
- #message ⇒ Object
- #model_message ⇒ Object
- #type_message ⇒ Object
Constructor Details
#initialize(resource_class, type: nil, model: nil) ⇒ PolymorphicResourceChildNotFound
Returns a new instance of PolymorphicResourceChildNotFound.
501 502 503 504 505 |
# File 'lib/graphiti/errors.rb', line 501 def initialize(resource_class, type: nil, model: nil) @resource_class = resource_class @model = model @type = type end |
Instance Method Details
#message ⇒ Object
507 508 509 |
# File 'lib/graphiti/errors.rb', line 507 def @model ? : end |
#model_message ⇒ Object
511 512 513 514 515 516 517 518 519 520 |
# File 'lib/graphiti/errors.rb', line 511 def <<~MSG #{@resource_class}: Tried to find Resource subclass with model #{@model.class}, but nothing found! Make sure all your child classes are assigned and associated to the right models: # One of these should be assocated to model #{@model.class}: self.polymorphic = ['Subclass1Resource', 'Subclass2Resource'] MSG end |
#type_message ⇒ Object
522 523 524 525 526 527 528 529 530 531 |
# File 'lib/graphiti/errors.rb', line 522 def <<~MSG #{@resource_class}: Tried to find Resource subclass with model #{@model.class}, but nothing found! Make sure all your child classes are assigned and associated to the right models: # One of these should be assocated to model #{@model.class}: self.polymorphic = ['Subclass1Resource', 'Subclass2Resource'] MSG end |