Exception: Graphiti::Errors::UndefinedIDLookup
- Defined in:
- lib/graphiti/errors.rb
Instance Method Summary collapse
-
#initialize(resource_class) ⇒ UndefinedIDLookup
constructor
A new instance of UndefinedIDLookup.
- #message ⇒ Object
Constructor Details
#initialize(resource_class) ⇒ UndefinedIDLookup
Returns a new instance of UndefinedIDLookup.
397 398 399 |
# File 'lib/graphiti/errors.rb', line 397 def initialize(resource_class) @resource_class = resource_class end |
Instance Method Details
#message ⇒ Object
401 402 403 404 405 406 407 |
# File 'lib/graphiti/errors.rb', line 401 def <<~MSG Tried to resolve #{@resource_class} with an :id filter, but the filter was nil. This can result in unscoping a query, which can cause incorrect values to be returned which may or may not bypass standard access controls. MSG end |