Class: Graphiti::Rails::FallbackHandler
- Inherits:
-
ExceptionHandler
- Object
- RescueRegistry::ExceptionHandler
- ExceptionHandler
- Graphiti::Rails::FallbackHandler
- Defined in:
- lib/graphiti/rails/exception_handlers.rb
Instance Method Summary collapse
-
#formatted_response(content_type, **options) ⇒ Object
A nil response is what hands the exception back to the default Rails handler, so formats Graphiti does not claim render as they always did.
Methods inherited from ExceptionHandler
Instance Method Details
#formatted_response(content_type, **options) ⇒ Object
A nil response is what hands the exception back to the default Rails handler, so formats Graphiti does not claim render as they always did.
48 49 50 51 52 |
# File 'lib/graphiti/rails/exception_handlers.rb', line 48 def formatted_response(content_type, **) return unless Graphiti::Rails.handled_exception_formats.include?(content_type.to_sym) super end |