Class: Graphiti::Util::InternalParam
- Defined in:
- lib/graphiti/util/internal_param.rb
Overview
Request params can only ever be strings, arrays and hashes, so a value wrapped in this class must have come from Graphiti itself.
Instance Attribute Summary collapse
-
#value ⇒ Object
readonly
Returns the value of attribute value.
Instance Method Summary collapse
- #blank? ⇒ Boolean
-
#initialize(value) ⇒ InternalParam
constructor
A new instance of InternalParam.
Constructor Details
#initialize(value) ⇒ InternalParam
Returns a new instance of InternalParam.
7 8 9 |
# File 'lib/graphiti/util/internal_param.rb', line 7 def initialize(value) @value = value end |
Instance Attribute Details
#value ⇒ Object (readonly)
Returns the value of attribute value.
5 6 7 |
# File 'lib/graphiti/util/internal_param.rb', line 5 def value @value end |
Instance Method Details
#blank? ⇒ Boolean
11 12 13 |
# File 'lib/graphiti/util/internal_param.rb', line 11 def blank? value.blank? end |